<?php
namespace Drupal\data_api;

class Data extends \AKlump\Data\Data
{
    protected $entityType = null;

    /**
     * @inheritdoc
     */
    public function get($subject, $path, $defaultValue = null, $valueCallback = null)
    {
        $this->cacheSet(__FUNCTION__, $subject, $path, $defaultValue, $valueCallback);
        if (empty($subject)) {
            return $this->postGet($defaultValue, $defaultValue, $valueCallback);
        }

        // This will make sure $path is an array.
        $this->validate($subject, $path);

        // When we know the entity type, we can use Drupal's field api functions.
        // This will take care of the translation.
        if (empty($this->cache['get']['level'])
            && ($field_name = reset($path))
            && ($bundle = $this->getBundleType($subject))
            && $this->isField($bundle, $field_name)
        ) {
            array_shift($path);
            if (!($subject = field_get_items($this->getEntityType(), $subject, $field_name))) {
                return $this->postGet($defaultValue, $defaultValue, $valueCallback);
            }
            elseif (count($path) === 0) {
                return $this->postGet($subject, $defaultValue, $valueCallback);
            }
        }

        return parent::get($subject, $path, $defaultValue, $valueCallback);
    }

    /**
     * Return the bundle type of the entity.
     *
     * @param string $subject
     *
     * @return null|bool
     */
    protected function getBundleType($subject)
    {
        $entity_type = $this->getEntityType();
        if (empty($entity_type)) {
            return null;
        }

        // We do not want to throw an exception here.
        try {
            list(, , $bundle) = entity_extract_ids($entity_type, $subject);
        } catch (\Exception $e) {
            $bundle = null;
        }

        return $bundle;
    }

    /**
     * @return string|null
     */
    protected function getEntityType()
    {
        return $this->entityType;
    }

    /**
     * @param string $entity_type
     *
     * @return $this
     */
    public function setEntityType($entity_type)
    {
        $this->entityType = $entity_type;

        return $this;
    }

    /**
     * Tests if $field is a field instance on $bundle.
     *
     * @param string $bundle
     * @param string $field
     *
     * @return bool
     */
    protected function isField($bundle, $field)
    {
        return array_key_exists($field, field_info_instances($this->getEntityType(), $bundle));
    }

    public function set(&$subject, $path, $value, $childTemplate = null)
    {
        $this->cacheSet(__FUNCTION__, $subject, $path, $value, $childTemplate);
        $this->validate($subject, $path);

        // Determine if we need to add the language component to the path.
        if (empty($this->cache['set']['level']) && count($path) > 1 && is_object($subject) && ($field_name = reset($path))) {
            $temp = clone $subject;;
            $temp->{$field_name} = isset($temp->{$field_name}) ? $temp->{$field_name} : $childTemplate;
            if (($bundle = $this->getBundleType($temp))
                && $this->isField($bundle, $field_name)
            ) {
                $langcode = field_language($this->getEntityType(), $subject, $field_name);
                if ($path[1] !== $langcode) {
                    array_splice($path, 1, 0, array($langcode));
                }
            }
        };

        return parent::set($subject, $path, $value, $childTemplate);
    }
}
